@@ -32,20 +32,25 @@
  */
 
 /**
- * Minimum PHP version; can't call PMA_fatalError() which uses a
- * PHP 5 function, so cannot easily localize this message.
+ * block attempts to directly run this script
  */
-if (version_compare(PHP_VERSION, '5.2.0', 'lt')) {
-    die('PHP 5.2+ is required');
+if (getcwd() == dirname(__FILE__)) {
+    die('Attack stopped');
 }
 
 /**
-  * Backward compatibility for PHP 5.2
-  */
-if (!defined('E_DEPRECATED')) {
-    define('E_DEPRECATED', 8192);
+ * Minimum PHP version; can't call PMA_fatalError() which uses a
+ * PHP 5 function, so cannot easily localize this message.
+ */
+if (version_compare(PHP_VERSION, '5.3.0', 'lt')) {
+    die('PHP 5.3+ is required');
 }
 
+/**
+ * for verification in all procedural scripts under libraries
+ */
+define('PHPMYADMIN', true);
+
 /**
  * the error handler
  */
@@ -57,17 +62,6 @@ require './libraries/Error_Handler.class.php';
 $GLOBALS['error_handler'] = new PMA_Error_Handler();
 $cfg['Error_Handler']['display'] = true;
 
-/*
- * This setting was removed in PHP 5.3. But at this point PMA_PHP_INT_VERSION
- * is not yet defined so we use another way to find out the PHP version.
- */
-if (version_compare(phpversion(), '5.3', 'lt')) {
-    /**
-     * Avoid object cloning errors
-     */
-    @ini_set('zend.ze1_compatibility_mode', false);
-}
-
 /**
  * This setting was removed in PHP 5.4. But at this point PMA_PHP_INT_VERSION
  * is not yet defined so we use another way to find out the PHP version.
@@ -75,15 +69,10 @@ if (version_compare(phpversion(), '5.3', 'lt')) {
 if (version_compare(phpversion(), '5.4', 'lt')) {
     /**
      * Avoid problems with magic_quotes_runtime
-     */ 
+     */
     @ini_set('magic_quotes_runtime', false);
 }
 
-/**
- * for verification in all procedural scripts under libraries
- */
-define('PHPMYADMIN', true);
-
 /**
  * core functions
  */
@@ -94,6 +83,13 @@ require './libraries/core.lib.php';
  */
 require './libraries/sanitizing.lib.php';
 
+/**
+ * Warning about mbstring.
+ */
+if (! function_exists('mb_detect_encoding')) {
+    PMA_warnMissingExtension('mbstring', $fatal = true);
+}
+
 /**
  * the PMA_Theme class
  */
@@ -124,14 +120,19 @@ require './libraries/Tracker.class.php';
  */
 require './libraries/Table.class.php';
 
-if (!defined('PMA_MINIMUM_COMMON')) {
+/**
+ * the PMA_Types class
+ */
+require './libraries/Types.class.php';
+
+if (! defined('PMA_MINIMUM_COMMON')) {
     /**
      * common functions
      */
-    include_once './libraries/common.lib.php';
+    include_once './libraries/Util.class.php';
 
     /**
-     * Java script escaping.
+     * JavaScript escaping.
      */
     include_once './libraries/js_escape.lib.php';
 
@@ -139,29 +140,16 @@ if (!defined('PMA_MINIMUM_COMMON')) {
      * Include URL/hidden inputs generating.
      */
     include_once './libraries/url_generating.lib.php';
+
+    /**
+     * Used to generate the page
+     */
+    include_once 'libraries/Response.class.php';
 }
 
 /******************************************************************************/
 /* start procedural code                       label_start_procedural         */
 
-/**
- * protect against possible exploits - there is no need to have so much variables
- */
-if (count($_REQUEST) > 1000) {
-    die(__('possible exploit'));
-}
-
-/**
- * Check for numeric keys
- * (if register_globals is on, numeric key can be found in $GLOBALS)
- */
-foreach ($GLOBALS as $key => $dummy) {
-    if (is_numeric($key)) {
-        die(__('numeric key detected'));
-    }
-}
-unset($dummy);
-
 /**
  * PATH_INFO could be compromised if set, so remove it from PHP_SELF
  * and provide a clean PHP_SELF here
@@ -242,7 +230,8 @@ if (isset($_POST['usesubform'])) {
      * track this
      */
     if (isset($_POST['redirect'])
-      && $_POST['redirect'] != basename($PMA_PHP_SELF)) {
+        && $_POST['redirect'] != basename($PMA_PHP_SELF)
+    ) {
         $__redirect = $_POST['redirect'];
         unset($_POST['redirect']);
     }
@@ -271,24 +260,18 @@ if (version_compare(phpversion(), '5.4', 'lt')) {
     }
 }
 
-/**
- * include deprecated grab_globals only if required
- */
-if (empty($__redirect) && !defined('PMA_NO_VARIABLES_IMPORT')) {
-    include './libraries/grab_globals.lib.php';
-}
-
 /**
  * check timezone setting
  * this could produce an E_STRICT - but only once,
  * if not done here it will produce E_STRICT on every date/time function
+ * (starting with PHP 5.3, this code can produce E_WARNING rather than
+ *  E_STRICT)
  *
- * @todo need to decide how we should handle this (without @)
  */
 date_default_timezone_set(@date_default_timezone_get());
 
 /******************************************************************************/
-/* parsing configuration file                         LABEL_parsing_config_file      */
+/* parsing configuration file                  LABEL_parsing_config_file      */
 
 /**
  * We really need this one!
@@ -297,6 +280,13 @@ if (! function_exists('preg_replace')) {
     PMA_warnMissingExtension('pcre', true);
 }
 
+/**
+ * JSON is required in several places.
+ */
+if (! function_exists('json_encode')) {
+    PMA_warnMissingExtension('json', true);
+}
+
 /**
  * @global PMA_Config $GLOBALS['PMA_Config']
  * force reading of config file, because we removed sensitive values
@@ -320,8 +310,9 @@ $GLOBALS['PMA_Config']->enableBc();
  */
 $pma_cookie_version = 4;
 if (isset($_COOKIE)
- && (isset($_COOKIE['pmaCookieVer'])
-  && $_COOKIE['pmaCookieVer'] < $pma_cookie_version)) {
+    && (isset($_COOKIE['pmaCookieVer'])
+    && $_COOKIE['pmaCookieVer'] < $pma_cookie_version)
+) {
     // delete all cookies
     foreach ($_COOKIE as $cookie_name => $tmp) {
         $GLOBALS['PMA_Config']->removeCookie($cookie_name);
@@ -335,11 +326,12 @@ if (isset($_COOKIE)
  * check HTTPS connection
  */
 if ($GLOBALS['PMA_Config']->get('ForceSSL')
-  && !$GLOBALS['PMA_Config']->get('is_https')) {
-    PMA_sendHeaderLocation(
-        preg_replace('/^http/', 'https',
-            $GLOBALS['PMA_Config']->get('PmaAbsoluteUri'))
-        . PMA_generate_common_url($_GET, 'text'));
+    && ! $GLOBALS['PMA_Config']->get('is_https')
+) {
+    // grab SSL URL
+    $url = $GLOBALS['PMA_Config']->getSSLUri();
+    // Actually redirect
+    PMA_sendHeaderLocation($url . PMA_URL_getCommon($_GET, 'text'));
     // delete the current session, otherwise we get problems (see bug #2397877)
     $GLOBALS['PMA_Config']->removeCookie($GLOBALS['session_name']);
     exit;
@@ -367,7 +359,6 @@ $GLOBALS['url_params'] = array();
  */
 $goto_whitelist = array(
     //'browse_foreigners.php',
-    //'calendar.php',
     //'changelog.php',
     //'chk_rel.php',
     'db_create.php',
@@ -383,18 +374,16 @@ $goto_whitelist = array(
     'db_printview.php',
     'db_search.php',
     'db_routines.php',
-    //'Documentation.html',
     'export.php',
     'import.php',
     //'index.php',
     //'navigation.php',
     //'license.php',
-    'main.php',
+    'index.php',
     'pdf_pages.php',
     'pdf_schema.php',
     //'phpinfo.php',
     'querywindow.php',
-    //'readme.php',
     'server_binlog.php',
     'server_collations.php',
     'server_databases.php',
@@ -402,13 +391,15 @@ $goto_whitelist = array(
     'server_export.php',
     'server_import.php',
     'server_privileges.php',
-    'server_processlist.php',
     'server_sql.php',
     'server_status.php',
+    'server_status_advisor.php',
+    'server_status_monitor.php',
+    'server_status_queries.php',
+    'server_status_variables.php',
     'server_variables.php',
     'sql.php',
     'tbl_addfield.php',
-    'tbl_alter.php',
     'tbl_change.php',
     'tbl_create.php',
     'tbl_import.php',
@@ -471,7 +462,12 @@ if (PMA_checkPageValidity($_REQUEST['back'], $goto_whitelist)) {
  * @todo variables should be handled by their respective owners (objects)
  * f.e. lang, server, collation_connection in PMA_Config
  */
-if (! PMA_isValid($_REQUEST['token']) || $_SESSION[' PMA_token '] != $_REQUEST['token']) {
+$token_mismatch = true;
+if (PMA_isValid($_REQUEST['token'])) {
+    $token_mismatch = ($_SESSION[' PMA_token '] != $_REQUEST['token']);
+}
+
+if ($token_mismatch) {
     /**
      *  List of parameters which are allowed from unsafe source
      */
@@ -486,11 +482,18 @@ if (! PMA_isValid($_REQUEST['token']) || $_SESSION[' PMA_token '] != $_REQUEST['
         'pma_lang', 'pma_collation_connection',
         /* Possible login form */
         'pma_servername', 'pma_username', 'pma_password',
-        /* for playing blobstreamable media */
-        'media_type', 'custom_type', 'bs_reference',
-        /* for changing BLOB repository file MIME type */
-        'bs_db', 'bs_table', 'bs_ref', 'bs_new_mime_type',
+        'recaptcha_challenge_field', 'recaptcha_response_field',
+        /* Needed to send the correct reply */
+        'ajax_request',
+        /* Permit to log out even if there is a token mismatch */
+        'old_usr'
     );
+    /**
+     * Allow changing themes in test/theme.php
+     */
+    if (defined('PMA_TEST_THEME')) {
+        $allow_list[] = 'set_theme';
+    }
     /**
      * Require cleanup functions
      */
@@ -498,7 +501,7 @@ if (! PMA_isValid($_REQUEST['token']) || $_SESSION[' PMA_token '] != $_REQUEST['
     /**
      * Do actual cleanup
      */
-    PMA_remove_request_vars($allow_list);
+    PMA_removeRequestVars($allow_list);
 
 }
 
@@ -548,49 +551,10 @@ if (PMA_isValid($_REQUEST['sql_query'])) {
     $GLOBALS['sql_query'] = $_REQUEST['sql_query'];
 }
 
-/**
- * avoid problems in phpmyadmin.css.php in some cases
- * @global string $js_frame
- */
-$_REQUEST['js_frame'] = PMA_ifSetOr($_REQUEST['js_frame'], '');
-
 //$_REQUEST['set_theme'] // checked later in this file LABEL_theme_setup
 //$_REQUEST['server']; // checked later in this file
 //$_REQUEST['lang'];   // checked by LABEL_loading_language_file
 
-
-/**
- * holds name of JavaScript files to be included in HTML header
- * @global array $js_include
- */
-$GLOBALS['js_include'] = array();
-$GLOBALS['js_include'][] = 'jquery/jquery-1.6.2.js';
-$GLOBALS['js_include'][] = 'jquery/jquery-ui-1.8.16.custom.js';
-$GLOBALS['js_include'][] = 'update-location.js';
-
-/**
- * holds an array of javascript code snippets to be included in the HTML header
- * Can be used with PMA_AddJSCode() to pass on js variables to the browser.
- * @global array $js_script
- */
-$GLOBALS['js_script'] = array();
-
-/**
- * Add common jQuery functions script here if necessary.
- */
-
-/**
- * JavaScript events that will be registered
- * @global array $js_events
- */
-$GLOBALS['js_events'] = array();
-
-/**
- * footnotes to be displayed ot the page bottom
- * @global array $footnotes
- */
-$GLOBALS['footnotes'] = array();
-
 /******************************************************************************/
 /* loading language file                       LABEL_loading_language_file    */
 
@@ -599,25 +563,41 @@ $GLOBALS['footnotes'] = array();
  */
 require './libraries/select_lang.lib.php';
 
+// Defines the cell alignment values depending on text direction
+if ($GLOBALS['text_dir'] == 'ltr') {
+    $GLOBALS['cell_align_left']  = 'left';
+    $GLOBALS['cell_align_right'] = 'right';
+} else {
+    $GLOBALS['cell_align_left']  = 'right';
+    $GLOBALS['cell_align_right'] = 'left';
+}
+
 /**
  * check for errors occurred while loading configuration
  * this check is done here after loading language files to present errors in locale
  */
+$GLOBALS['PMA_Config']->checkPermissions();
+
 if ($GLOBALS['PMA_Config']->error_config_file) {
-    $error = '<h1>' . __('Failed to read configuration file') . '</h1>'
-        . _('This usually means there is a syntax error in it, please check any errors shown below.')
-        . '<br />'
-        . '<br />'
-        . '<iframe src="show_config_errors.php" />';
+    $error = '[strong]' . __('Failed to read configuration file') . '[/strong]'
+        . '[br][br]'
+        . __('This usually means there is a syntax error in it, please check any errors shown below.')
+        . '[br][br]'
+        . '[conferr]';
     trigger_error($error, E_USER_ERROR);
 }
 if ($GLOBALS['PMA_Config']->error_config_default_file) {
-    $error = sprintf(__('Could not load default configuration from: %1$s'),
-        $GLOBALS['PMA_Config']->default_source);
+    $error = sprintf(
+        __('Could not load default configuration from: %1$s'),
+        $GLOBALS['PMA_Config']->default_source
+    );
     trigger_error($error, E_USER_ERROR);
 }
 if ($GLOBALS['PMA_Config']->error_pma_uri) {
-    trigger_error(__('The <tt>$cfg[\'PmaAbsoluteUri\']</tt> directive MUST be set in your configuration file!'), E_USER_ERROR);
+    trigger_error(
+        __('The [code]$cfg[\'PmaAbsoluteUri\'][/code] directive MUST be set in your configuration file!'),
+        E_USER_ERROR
+    );
 }
 
 
@@ -647,14 +627,23 @@ if (! isset($cfg['Servers']) || count($cfg['Servers']) == 0) {
 
         // Detect wrong configuration
         if (!is_int($server_index) || $server_index < 1) {
-            trigger_error(sprintf(__('Invalid server index: %s'), $server_index), E_USER_ERROR);
+            trigger_error(
+                sprintf(__('Invalid server index: %s'), $server_index),
+                E_USER_ERROR
+            );
         }
 
         $each_server = array_merge($default_server, $each_server);
 
         // Don't use servers with no hostname
         if ($each_server['connect_type'] == 'tcp' && empty($each_server['host'])) {
-            trigger_error(sprintf(__('Invalid hostname for server %1$s. Please review your configuration.'), $server_index), E_USER_ERROR);
+            trigger_error(
+                sprintf(
+                    __('Invalid hostname for server %1$s. Please review your configuration.'),
+                    $server_index
+                ),
+                E_USER_ERROR
+            );
         }
 
         // Final solution to bug #582890
@@ -662,8 +651,11 @@ if (! isset($cfg['Servers']) || count($cfg['Servers']) == 0) {
         // and there is nothing in the verbose server name
         // or the host field, then generate a name for the server
         // in the form of "Server 2", localized of course!
-        if ($each_server['connect_type'] == 'socket' && empty($each_server['host']) && empty($each_server['verbose'])) {
-            $each_server['verbose'] = __('Server') . $server_index;
+        if ($each_server['connect_type'] == 'socket'
+            && empty($each_server['host'])
+            && empty($each_server['verbose'])
+        ) {
+            $each_server['verbose'] = sprintf(__('Server %d'), $server_index);
         }
 
         $new_servers[$server_index] = $each_server;
@@ -737,13 +729,6 @@ $GLOBALS['pmaThemeImage']   = $_SESSION['PMA_Theme']->getImgPath();
  */
 if (@file_exists($_SESSION['PMA_Theme']->getLayoutFile())) {
     include $_SESSION['PMA_Theme']->getLayoutFile();
-    /**
-     * @todo remove if all themes are update use Navi instead of Left as frame name
-     */
-    if (! isset($GLOBALS['cfg']['NaviWidth'])
-     && isset($GLOBALS['cfg']['LeftWidth'])) {
-        $GLOBALS['cfg']['NaviWidth'] = $GLOBALS['cfg']['LeftWidth'];
-    }
 }
 
 if (! defined('PMA_MINIMUM_COMMON')) {
@@ -755,16 +740,22 @@ if (! defined('PMA_MINIMUM_COMMON')) {
     /**
      * String handling
      */
-    include_once './libraries/string.lib.php';
+    include_once './libraries/string.inc.php';
 
     /**
      * Lookup server by name
      * (see FAQ 4.8)
      */
-    if (! empty($_REQUEST['server']) && is_string($_REQUEST['server'])
-     && ! is_numeric($_REQUEST['server'])) {
+    if (! empty($_REQUEST['server'])
+        && is_string($_REQUEST['server'])
+        && ! is_numeric($_REQUEST['server'])
+    ) {
         foreach ($cfg['Servers'] as $i => $server) {
-            if ($server['host'] == $_REQUEST['server']) {
+            if ($server['host'] == $_REQUEST['server']
+                || $server['verbose'] == $_REQUEST['server']
+                || $PMA_String->strtolower($server['verbose']) == $PMA_String->strtolower($_REQUEST['server'])
+                || md5($PMA_String->strtolower($server['verbose'])) == $PMA_String->strtolower($_REQUEST['server'])
+            ) {
                 $_REQUEST['server'] = $i;
                 break;
             }
@@ -784,7 +775,11 @@ if (! defined('PMA_MINIMUM_COMMON')) {
      * and '$cfg['ServerDefault'] = 0' is set.
      */
 
-    if (isset($_REQUEST['server']) && (is_string($_REQUEST['server']) || is_numeric($_REQUEST['server'])) && ! empty($_REQUEST['server']) && ! empty($cfg['Servers'][$_REQUEST['server']])) {
+    if (isset($_REQUEST['server'])
+        && (is_string($_REQUEST['server']) || is_numeric($_REQUEST['server']))
+        && ! empty($_REQUEST['server'])
+        && ! empty($cfg['Servers'][$_REQUEST['server']])
+    ) {
         $GLOBALS['server'] = $_REQUEST['server'];
         $cfg['Server'] = $cfg['Servers'][$GLOBALS['server']];
     } else {
@@ -802,7 +797,8 @@ if (! defined('PMA_MINIMUM_COMMON')) {
      * Kanji encoding convert feature appended by Y.Kawada (2002/2/20)
      */
     if (function_exists('mb_convert_encoding')
-     && $lang == 'ja') {
+        && $lang == 'ja'
+    ) {
         include_once './libraries/kanji-encoding.lib.php';
     } // end if
 
@@ -813,8 +809,9 @@ if (! defined('PMA_MINIMUM_COMMON')) {
     $GLOBALS['PMA_Config']->setCookie('pma_lang', $GLOBALS['lang']);
     if (isset($GLOBALS['collation_connection'])) {
         $GLOBALS['PMA_Config']->setCookie(
-            'pma_collation_connection', 
-            $GLOBALS['collation_connection']);
+            'pma_collation_connection',
+            $GLOBALS['collation_connection']
+        );
     }
 
     $_SESSION['PMA_Theme_Manager']->setThemeCookie();
@@ -824,13 +821,13 @@ if (! defined('PMA_MINIMUM_COMMON')) {
         /**
          * Loads the proper database interface for this server
          */
-        include_once './libraries/database_interface.lib.php';
+        include_once './libraries/database_interface.inc.php';
 
         include_once './libraries/logging.lib.php';
 
         // get LoginCookieValidity from preferences cache
-        // no generic solution for loading preferences from cache as some settings need to be kept
-        // for processing in PMA_Config::loadUserPreferences()
+        // no generic solution for loading preferences from cache as some settings
+        // need to be kept for processing in PMA_Config::loadUserPreferences()
         $cache_key = 'server_' . $GLOBALS['server'];
         if (isset($_SESSION['cache'][$cache_key]['userprefs']['LoginCookieValidity'])) {
             $value = $_SESSION['cache'][$cache_key]['userprefs']['LoginCookieValidity'];
@@ -845,19 +842,31 @@ if (! defined('PMA_MINIMUM_COMMON')) {
 
         // to allow HTTP or http
         $cfg['Server']['auth_type'] = strtolower($cfg['Server']['auth_type']);
-        if (! file_exists('./libraries/auth/' . $cfg['Server']['auth_type'] . '.auth.lib.php')) {
-            PMA_fatalError(__('Invalid authentication method set in configuration:') . ' ' . $cfg['Server']['auth_type']);
-        }
+
         /**
          * the required auth type plugin
          */
-        include_once './libraries/auth/' . $cfg['Server']['auth_type'] . '.auth.lib.php';
-        if (!PMA_auth_check()) {
+        $auth_class = "Authentication" . ucfirst($cfg['Server']['auth_type']);
+        if (! file_exists(
+            './libraries/plugins/auth/'
+            . $auth_class . '.class.php'
+        )) {
+            PMA_fatalError(
+                __('Invalid authentication method set in configuration:')
+                . ' ' . $cfg['Server']['auth_type']
+            );
+        }
+        include_once  './libraries/plugins/auth/' . $auth_class . '.class.php';
+        // todo: add plugin manager
+        $plugin_manager = null;
+        $auth_plugin = new $auth_class($plugin_manager);
+
+        if (! $auth_plugin->authCheck()) {
             /* Force generating of new session on login */
             PMA_secureSession();
-            PMA_auth();
+            $auth_plugin->auth();
         } else {
-            PMA_auth_set_user();
+            $auth_plugin->authSetUser();
         }
 
          // Check IP-based Allow/Deny rules as soon as possible to reject the
@@ -866,7 +875,8 @@ if (! defined('PMA_MINIMUM_COMMON')) {
         // http://cvs.apache.org/viewcvs.cgi/httpd-2.0/modules/aaa/mod_access.c?rev=1.37&content-type=text/vnd.viewcvs-markup
         // Look at: "static int check_dir_access(request_rec *r)"
         if (isset($cfg['Server']['AllowDeny'])
-                && isset($cfg['Server']['AllowDeny']['order'])) {
+            && isset($cfg['Server']['AllowDeny']['order'])
+        ) {
 
             /**
              * ip based access library
@@ -890,8 +900,7 @@ if (! defined('PMA_MINIMUM_COMMON')) {
                     $allowDeny_forbidden = false;
                 }
             } elseif ($cfg['Server']['AllowDeny']['order'] == 'explicit') {
-                if (PMA_allowDeny('allow')
-                  && !PMA_allowDeny('deny')) {
+                if (PMA_allowDeny('allow') && ! PMA_allowDeny('deny')) {
                     $allowDeny_forbidden = false;
                 } else {
                     $allowDeny_forbidden = true;
@@ -900,26 +909,25 @@ if (! defined('PMA_MINIMUM_COMMON')) {
 
             // Ejects the user if banished
             if ($allowDeny_forbidden) {
-                PMA_log_user($cfg['Server']['user'], 'allow-denied');
-                PMA_auth_fails();
+                PMA_logUser($cfg['Server']['user'], 'allow-denied');
+                $auth_plugin->authFails();
             }
-            unset($allowDeny_forbidden); //Clean up after you!
         } // end if
 
         // is root allowed?
-        if (!$cfg['Server']['AllowRoot'] && $cfg['Server']['user'] == 'root') {
+        if (! $cfg['Server']['AllowRoot'] && $cfg['Server']['user'] == 'root') {
             $allowDeny_forbidden = true;
-            PMA_log_user($cfg['Server']['user'], 'root-denied');
-            PMA_auth_fails();
-            unset($allowDeny_forbidden); //Clean up after you!
+            PMA_logUser($cfg['Server']['user'], 'root-denied');
+            $auth_plugin->authFails();
         }
 
         // is a login without password allowed?
-        if (!$cfg['Server']['AllowNoPassword'] && $cfg['Server']['password'] == '') {
+        if (! $cfg['Server']['AllowNoPassword']
+            && $cfg['Server']['password'] == ''
+        ) {
             $login_without_password_is_forbidden = true;
-            PMA_log_user($cfg['Server']['user'], 'empty-denied');
-            PMA_auth_fails();
-            unset($login_without_password_is_forbidden); //Clean up after you!
+            PMA_logUser($cfg['Server']['user'], 'empty-denied');
+            $auth_plugin->authFails();
         }
 
         // if using TCP socket is not needed
@@ -933,27 +941,52 @@ if (! defined('PMA_MINIMUM_COMMON')) {
         // scripts)
         $controllink = false;
         if ($cfg['Server']['controluser'] != '') {
-            if (! empty($cfg['Server']['controlhost'])) {
-                $controllink = PMA_DBI_connect($cfg['Server']['controluser'],
-                    $cfg['Server']['controlpass'], true,
-                    array('host' => $cfg['Server']['controlhost'])
+            if (! empty($cfg['Server']['controlhost'])
+                || ! empty($cfg['Server']['controlport'])
+            ) {
+                $server_details = array();
+                if (! empty($cfg['Server']['controlhost'])) {
+                    $server_details['host'] = $cfg['Server']['controlhost'];
+                } else {
+                    $server_details['host'] = $cfg['Server']['host'];
+                }
+                if (! empty($cfg['Server']['controlport'])) {
+                    $server_details['port'] = $cfg['Server']['controlport'];
+                } elseif ($server_details['host'] == $cfg['Server']['host']) {
+                    // Evaluates to true when controlhost == host
+                    // or controlhost is not defined (hence it defaults to host)
+                    // In such case we can use the value of port.
+                    $server_details['port'] = $cfg['Server']['port'];
+                }
+                // otherwise we leave the $server_details['port'] unset,
+                // allowing it to take default mysql port
+
+                $controllink = $GLOBALS['dbi']->connect(
+                    $cfg['Server']['controluser'],
+                    $cfg['Server']['controlpass'],
+                    true,
+                    $server_details
                 );
             } else {
-                $controllink = PMA_DBI_connect($cfg['Server']['controluser'],
-                    $cfg['Server']['controlpass'], true);
+                $controllink = $GLOBALS['dbi']->connect(
+                    $cfg['Server']['controluser'],
+                    $cfg['Server']['controlpass'],
+                    true
+                );
             }
         }
 
         // Connects to the server (validates user's login)
-        $userlink = PMA_DBI_connect($cfg['Server']['user'],
-            $cfg['Server']['password'], false);
+        $userlink = $GLOBALS['dbi']->connect(
+            $cfg['Server']['user'], $cfg['Server']['password'], false
+        );
 
         if (! $controllink) {
             $controllink = $userlink;
         }
 
         /* Log success */
-        PMA_log_user($cfg['Server']['user']);
+        PMA_logUser($cfg['Server']['user']);
 
         /**
          * with phpMyAdmin 3 we support MySQL >=5
@@ -961,13 +994,22 @@ if (! defined('PMA_MINIMUM_COMMON')) {
          *  - > 5.0.15
          */
         if (PMA_MYSQL_INT_VERSION < 50015) {
-            PMA_fatalError(__('You should upgrade to %s %s or later.'), array('MySQL', '5.0.15'));
+            PMA_fatalError(
+                __('You should upgrade to %s %s or later.'),
+                array('MySQL', '5.0.15')
+            );
+        }
+
+        /**
+         * Type handling object.
+         */
+        if (PMA_DRIZZLE) {
+            $GLOBALS['PMA_Types'] = new PMA_Types_Drizzle();
+        } else {
+            $GLOBALS['PMA_Types'] = new PMA_Types_MySQL();
         }
 
         if (PMA_DRIZZLE) {
-            // DisableIS must be set to false for Drizzle, it maps SHOW commands
-            // to INFORMATION_SCHEMA queries anyway so it's fast on large servers
-            $cfg['Server']['DisableIS'] = false;
             // SHOW OPEN TABLES is not supported by Drizzle
             $cfg['SkipLockedTables'] = false;
         }
@@ -993,10 +1035,12 @@ if (! defined('PMA_MINIMUM_COMMON')) {
         /**
          * some resetting has to be done when switching servers
          */
-        if (isset($_SESSION['tmp_user_values']['previous_server']) && $_SESSION['tmp_user_values']['previous_server'] != $GLOBALS['server']) {
-            unset($_SESSION['tmp_user_values']['navi_limit_offset']);
+        if (isset($_SESSION['tmpval']['previous_server'])
+            && $_SESSION['tmpval']['previous_server'] != $GLOBALS['server']
+        ) {
+            unset($_SESSION['tmpval']['navi_limit_offset']);
         }
-        $_SESSION['tmp_user_values']['previous_server'] = $GLOBALS['server'];
+        $_SESSION['tmpval']['previous_server'] = $GLOBALS['server'];
 
     } // end server connecting
 
@@ -1004,20 +1048,43 @@ if (! defined('PMA_MINIMUM_COMMON')) {
      * check if profiling was requested and remember it
      * (note: when $cfg['ServerDefault'] = 0, constant is not defined)
      */
-    if (isset($_REQUEST['profiling']) && PMA_profilingSupported()) {
+    if (isset($_REQUEST['profiling'])
+        && PMA_Util::profilingSupported()
+    ) {
         $_SESSION['profiling'] = true;
     } elseif (isset($_REQUEST['profiling_form'])) {
         // the checkbox was unchecked
         unset($_SESSION['profiling']);
     }
+    /**
+     * Inclusion of profiling scripts is needed on various
+     * pages like sql, tbl_sql, db_sql, tbl_select
+     */
+    if (! defined('PMA_BYPASS_GET_INSTANCE')) {
+        $response = PMA_Response::getInstance();
+    }
+    if (isset($_SESSION['profiling'])) {
+        $header   = $response->getHeader();
+        $scripts  = $header->getScripts();
+        $scripts->addFile('jqplot/jquery.jqplot.js');
+        $scripts->addFile('jqplot/plugins/jqplot.pieRenderer.js');
+        $scripts->addFile('jqplot/plugins/jqplot.highlighter.js');
+        $scripts->addFile('canvg/canvg.js');
+        $scripts->addFile('jquery/jquery.tablesorter.js');
+    }
 
-    // library file for blobstreaming
-    include_once './libraries/blobstreaming.lib.php';
-
-    // checks for blobstreaming plugins and databases that support
-    // blobstreaming (by having the necessary tables for blobstreaming)
-    checkBLOBStreamingPlugins();
-
+    /*
+     * There is no point in even attempting to process
+     * an ajax request if there is a token mismatch
+     */
+    if (isset($response) && $response->isAjax() && $token_mismatch) {
+        $response->isSuccess(false);
+        $response->addJSON(
+            'message',
+            PMA_Message::error(__('Error: Token mismatch'))
+        );
+        exit;
+    }
 } // end if !defined('PMA_MINIMUM_COMMON')
 
 // load user preferences
@@ -1045,18 +1112,33 @@ if (isset($_REQUEST['ajax_request']) && $_REQUEST['ajax_request'] == true) {
     $GLOBALS['is_ajax_request'] = false;
 }
 
-/**
- * @global  boolean $GLOBALS['grid_edit']
- *
- * Set to true if this is a request made during an grid edit process.  This
- * request is made to retrieve the non-truncated/transformed values.
- */
-if (isset($_REQUEST['grid_edit']) && $_REQUEST['grid_edit'] == true) {
-    $GLOBALS['grid_edit'] = true;
-} else {
-    $GLOBALS['grid_edit'] = false;
+if (isset($_REQUEST['GLOBALS']) || isset($_FILES['GLOBALS'])) {
+    PMA_fatalError(__("GLOBALS overwrite attempt"));
 }
 
+/**
+ * protect against possible exploits - there is no need to have so much variables
+ */
+if (count($_REQUEST) > 1000) {
+    PMA_fatalError(__('possible exploit'));
+}
+
+/**
+ * Check for numeric keys
+ * (if register_globals is on, numeric key can be found in $GLOBALS)
+ */
+foreach ($GLOBALS as $key => $dummy) {
+    if (is_numeric($key)) {
+        PMA_fatalError(__('numeric key detected'));
+    }
+}
+unset($dummy);
+
+// here, the function does not exist with this configuration:
+// $cfg['ServerDefault'] = 0;
+$GLOBALS['is_superuser']
+    = isset($GLOBALS['dbi']) && $GLOBALS['dbi']->isSuperuser();
+
 if (!empty($__redirect) && in_array($__redirect, $goto_whitelist)) {
     /**
      * include subform target page
@@ -1064,4 +1146,5 @@ if (!empty($__redirect) && in_array($__redirect, $goto_whitelist)) {
     include $__redirect;
     exit();
 }
+
 ?>
